package com.gic.enterprise.constant.applet;
/**
 * 小程序自定义页面装修页面类型
 * @ClassName: AppletPageTypeEnum 
 * @Description:  
 * @author guojuxing 
 * @date 2019/11/25 2:42 PM 
 */
public enum AppletPageTypeEnum {
    /**
     * 自定义页面
     */
    CUSTOM(1, "自定义页面", 1, 0),
    /**
     * 会员中心
     */
    MEMBER_CENTER(2, "会员中心", 2, 1),
    /**
     * 客服服务页面
     */
    CUSTOMER_SERVICE(3, "客服服务页面", 2, 0),
    /**
     * 积分服务
     */
    INTEGRAL_SERVICE(4, "积分服务", 1, 0),
    /**
     * 积分成就
     */
    MEMBER_ACHIEVEMENT(5, "积分成就", 2, 0),
    /**
     * 会员任务
     */
    MEMBER_TASK(6, "会员任务", 2, 0),
    /**
     * 卡券详情列表
     */
    CARD_DETAIL_LIST(7, "卡券详情列表", 2, 0),
    /**
     * 商城首页
     */
    MALL_HOME_PAGE(8, "商城首页", 2, 0),
    /**
     * 商城分类
     */
    MALL_CLASSIFY(9, "商城分类", 2, 0);

    private int code;
    private String message;
    /**
     * 发布状态类型 2：只有一条的 1：有多条的
     */
    private int type;
    /**
     * 是否有默认发布状态的 1：是 0：不是
     */
    private int hasDefault;

    /**
     * 是否是枚举内的code值
     * @param code
     * @return
     */
    public static boolean isRightCode(Integer code) {
        if (code == null) {
            return false;
        }
        for (AppletPageTypeEnum typeEnum : values()) {
            if (code.intValue() == typeEnum.getCode()) {
                return true;
            }
        }
        return false;
    }

    /**
     * 发布状态只有一条的
     * @param code
     * @return
     */
    public static boolean isOnlyOneCode(Integer code) {
        if (code == null) {
            return false;
        }
        for (AppletPageTypeEnum typeEnum : values()) {
            if (code.intValue() == typeEnum.getCode() && typeEnum.getType() == 2) {
                return true;
            }
        }
        return false;
    }

    /**
     * 是否有默认记录的页面
     * @param code
     * @return
     */
    public static boolean hasDefault(Integer code) {
        if (code == null) {
            return false;
        }
        for (AppletPageTypeEnum typeEnum : values()) {
            if (code.intValue() == typeEnum.getCode() && typeEnum.getHasDefault() == 1) {
                return true;
            }
        }
        return false;
    }

    /**
     * 是否需要设置页面背景字段的页面
     * @param code
     * @return
     */
    public static boolean hasBackgroundColor(Integer code) {
        if (code == null) {
            return false;
        }
        if (MEMBER_CENTER.getCode() == code.intValue() ||
                CUSTOM.getCode() == code.intValue() ||
                INTEGRAL_SERVICE.getCode() == code.intValue() ||
                MALL_HOME_PAGE.getCode() == code.intValue()) {
            return true;
        }
        return false;
    }

    /**
     * 是否是有人群筛选的页面
     * @param pageType
     * @return
     */
    public static boolean needCrowdWidget(Integer pageType) {
        if (pageType == null) {
            return false;
        }
        if (AppletPageTypeEnum.INTEGRAL_SERVICE.getCode() == pageType) {
            return true;
        }
        return false;
    }

    private AppletPageTypeEnum(int code, String message, int type, int hasDefault) {
        this.code = code;
        this.message = message;
        this.type = type;
        this.hasDefault = hasDefault;
    }

    public int getCode() {
        return code;
    }

    public void setCode(int code) {
        this.code = code;
    }

    public String getMessage() {
        return message;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    public int getType() {
        return type;
    }

    public void setType(int type) {
        this.type = type;
    }

    public int getHasDefault() {
        return hasDefault;
    }

    public void setHasDefault(int hasDefault) {
        this.hasDefault = hasDefault;
    }
}
