package com.gic.enterprise.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ctrip.framework.apollo.Config;
import com.ctrip.framework.apollo.ConfigService;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.ToolUtil;
import com.gic.dsmongo.api.constant.enums.MongoOperTypeEnum;
import com.gic.dsmongo.api.dto.Criteria;
import com.gic.dsmongo.api.dto.MongoQueryDTO;
import com.gic.dsmongo.api.dto.MongoSaveDTO;
import com.gic.dsmongo.api.dto.MongoUpdateDTO;
import com.gic.dsmongo.api.service.MongoOperApiService;
import com.gic.dubbo.util.DubboContextUtil;
import com.gic.dubbo.util.DubboInvokeUtil;
import com.gic.enterprise.constant.FieldTypeEnum;
import com.gic.enterprise.dto.TableSettingDTO;
import com.gic.enterprise.dto.TableSettingFieldDTO;
import com.gic.enterprise.entity.CustomSettingData;
import com.gic.enterprise.entity.TabTableSetting;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.CustomSettingApiService;
import com.gic.enterprise.service.TableSettingFieldService;
import com.gic.enterprise.service.TableSettingService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.dubbo.rpc.service.GenericService;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service("customSettingApiService")
public class CustomSettingApiServiceImpl implements CustomSettingApiService {
    private static final Logger log = LogManager.getLogger(CustomSettingApiServiceImpl.class);
    private static final String DATABASENAME = "custom_table";
    private static final String PRIMARYKEY = "id";
    @Autowired
    private MongoOperApiService mongoOperApiService;
    @Autowired
    TableSettingService tableSettingService;
    @Autowired
    TableSettingFieldService tableSettingFieldService;

    @Override
    public ServiceResponse<String> save(String tableName, String params) {
        log.info("保存参数：{}，{}", tableName, params);
        TabTableSetting tableSetting = tableSettingService.getTableSetting(tableName);
        if(tableSetting == null){
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), tableName + "不存在");
        }
        Map<String, Object> map = JSON.parseObject(params, Map.class);
        String repeateSetting = tableSetting.getRepeateSetting();
        if(StringUtils.isNotBlank(repeateSetting)){
            String[] split = repeateSetting.split(",");
            List<Criteria> criteriaList = new ArrayList<>();
            for(String field : split){
                Criteria criteria = new Criteria(field, MongoOperTypeEnum.EQ, map.get(field));
                criteriaList.add(criteria);
            }
            if(map.get(PRIMARYKEY) != null){
                Criteria criteria = new Criteria(PRIMARYKEY, MongoOperTypeEnum.NIN, map.get(PRIMARYKEY));
                criteriaList.add(criteria);
            }
            MongoQueryDTO queryDTO = new MongoQueryDTO();
            queryDTO.setCriteria(criteriaList);
            queryDTO.setDataBaseName(DATABASENAME);
            queryDTO.setConllectionName(tableName);
            ServiceResponse<List<Map>> query = mongoOperApiService.query(queryDTO);
            log.info("是否存在重复:{},{}", JSON.toJSONString(queryDTO), JSON.toJSONString(query));
            if(CollectionUtils.isNotEmpty(query.getResult())){
                if(tableSetting.getRepeateOpt() != null && tableSetting.getRepeateOpt() == 1){
                    //等于1表示重复更新
                    Map exitMap = query.getResult().get(0);
                    map.forEach((k, v) -> exitMap.put(k, v));
                    map = exitMap;
                } else {
                    return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "存在重复记录");
                }
            }
        }
        log.info("params:{}", JSON.toJSONString(map));
        Date date = new Date();
        if(map.get(PRIMARYKEY) != null){
            //更新
            map.put("update_time", date);
            MongoUpdateDTO mongoUpdateDTO = new MongoUpdateDTO();
            mongoUpdateDTO.setDataBaseAndCollectionName(DATABASENAME, tableName, PRIMARYKEY);
            mongoUpdateDTO.setData(map);
            ServiceResponse<Integer> update = mongoOperApiService.update(mongoUpdateDTO);
            log.info("update:{},{}", JSON.toJSONString(map), JSON.toJSONString(update));
            if(update.isSuccess()){
                String saveExtInterface = tableSetting.getSaveExtInterface();
                if(StringUtils.isNotBlank(saveExtInterface)){
                    extendInterface(saveExtInterface, params);
                }
                return ServiceResponse.success();
            } else {
                return ServiceResponse.failure(update.getCode(), update.getMessage());
            }
        } else {
            map.put("update_time", date);
            map.put("create_time", date);
            map.put("delete_flag", 0);
            MongoSaveDTO mongoSaveDTO = new MongoSaveDTO();
            map.put(PRIMARYKEY, ToolUtil.randomUUID());
            mongoSaveDTO.setDataBaseAndCollectionName(DATABASENAME, tableName, PRIMARYKEY);
            mongoSaveDTO.setData(map);
            ServiceResponse<String> save = mongoOperApiService.save(mongoSaveDTO);
            log.info("save:{}", JSON.toJSONString(save));
            if(save.isSuccess()){
                String saveExtInterface = tableSetting.getSaveExtInterface();
                if(StringUtils.isNotBlank(saveExtInterface)){
                    extendInterface(saveExtInterface, params);
                }
                return ServiceResponse.success();
            } else {
                return ServiceResponse.failure(save.getCode(), save.getMessage());
            }
        }
    }


    @Override
    public ServiceResponse<Map<String, Object>> getDetail(String tableName, String searchParams) {
        Map<String, Object> map = JSON.parseObject(searchParams, Map.class);
        MongoQueryDTO queryDTO = new MongoQueryDTO();
        if(MapUtils.isNotEmpty(map)){
            TabTableSetting tableSetting = tableSettingService.getTableSetting(tableName);
            if(tableSetting == null){
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), tableName + "不存在");
            }
            List<Criteria> criteriaList = getCriteriaList(map, tableSetting);
            queryDTO.setCriteria(criteriaList);
        }
        queryDTO.setDataBaseName(DATABASENAME);
        queryDTO.setConllectionName(tableName);
        ServiceResponse<List<Map>> query = mongoOperApiService.query(queryDTO);
        if(query.isSuccess()){
            if(CollectionUtils.isNotEmpty(query.getResult())){
                return ServiceResponse.success(query.getResult().get(0));
            } else {
                return ServiceResponse.success();
            }
        } else {
            return ServiceResponse.failure(query.getCode(), query.getMessage());
        }
    }

    @Override
    public ServiceResponse<Page<Map>> page(String tableName, String searchParams, Integer currentPage, Integer pageSize) {
        Map<String, Object> map = JSON.parseObject(searchParams, Map.class);
        MongoQueryDTO queryDTO = new MongoQueryDTO();
        if(MapUtils.isNotEmpty(map)){
            TabTableSetting tableSetting = tableSettingService.getTableSetting(tableName);
            if(tableSetting == null){
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), tableName + "不存在");
            }
            List<Criteria> criteriaList = getCriteriaList(map, tableSetting);
            queryDTO.setCriteria(criteriaList);
        }
        queryDTO.setDataBaseName(DATABASENAME);
        queryDTO.setConllectionName(tableName);
        queryDTO.setCurrentPage(currentPage);
        queryDTO.setPageSize(pageSize);
        ServiceResponse<Page<Map>> pageServiceResponse = mongoOperApiService.queryByPage(queryDTO);
        return pageServiceResponse;
    }

    @Override
    public ServiceResponse<Integer> delete(String tableName, String id) {
        ServiceResponse<Map<String, Object>> detail = getDetail(tableName, "{\"id\":\"" + id + "\"}");
        Map<String, Object> map = detail.getResult();
        map.put("update_time", new Date());
        map.put("delete_flag", 1);
        MongoUpdateDTO mongoUpdateDTO = new MongoUpdateDTO();
        mongoUpdateDTO.setDataBaseAndCollectionName(DATABASENAME, tableName, PRIMARYKEY);
        mongoUpdateDTO.setData(map);
        ServiceResponse<Integer> update = mongoOperApiService.update(mongoUpdateDTO);
        if(update.isSuccess()){
            TabTableSetting tableSetting = tableSettingService.getTableSetting(tableName);
            if(StringUtils.isNotBlank(tableSetting.getDelExtInterface())){
                extendInterface(tableSetting.getDelExtInterface(), JSON.toJSONString(map));
            }
        }
        return update;
    }


    private List<Criteria> getCriteriaList(Map<String, Object> map, TabTableSetting tableSetting){
        List<TableSettingFieldDTO> fieldDTOS = tableSettingFieldService.listTableField(tableSetting.getTableId());
        Map<String, TableSettingFieldDTO> fieldDTOMap = fieldDTOS.stream().collect(Collectors.toMap(TableSettingFieldDTO::getFieldKey, TableSettingFieldDTO -> TableSettingFieldDTO));
        List<Criteria> criteriaList = new ArrayList<>();
        map.forEach((k, v) -> {
            Criteria criteria = null;
            if(k.equals(PRIMARYKEY)){
                criteria = new Criteria(k, MongoOperTypeEnum.EQ, v);
            } else {
                if(fieldDTOMap.get(k).getFieldType().equals(FieldTypeEnum.STRING.getType())){
                    criteria = new Criteria(k, MongoOperTypeEnum.REGEX, v);
                } else if(fieldDTOMap.get(k).getFieldType().equals(FieldTypeEnum.DATE.getType())){
                    String s = v+"";
                    if(StringUtils.isNotBlank(s)){
                        String[] split = s.split("-");
                        if(split.length == 2){
                            JSONObject json = new JSONObject();
                            json.put("start", split[0]);
                            json.put("end", split[1]);
                            criteria = new Criteria(k, MongoOperTypeEnum.BETWEEN, json);
                        }
                    }
                } else {
                    criteria = new Criteria(k, MongoOperTypeEnum.EQ, v);
                }
            }
            if(criteria != null){
                criteriaList.add(criteria);
            }
        });
        Criteria criteria = new Criteria("delete_flag", MongoOperTypeEnum.EQ, 1);
        criteriaList.add(criteria);
        return criteriaList;
    }


    private void extendInterface(String extInterface, String param){
        Config config = ConfigService.getConfig("COMMON.4.0-dubbo");
        String zookeeperHost = config.getProperty("zookeeper_host", "");
        String host = zookeeperHost.substring(zookeeperHost.indexOf("//")+2, zookeeperHost.lastIndexOf(":"));
        String port = zookeeperHost.substring(zookeeperHost.lastIndexOf(":")+1);
        String service = extInterface.substring(0, extInterface.lastIndexOf("."));
        String method = extInterface.substring(extInterface.lastIndexOf(".")+1);
        DubboContextUtil.initRegistryConfig(host, Integer.valueOf(port));
        GenericService genericService = DubboInvokeUtil.getGenericService(service);
        Object o = genericService.$invoke(method, new String[]{String.class.getName()}, new String[]{param});
    }
}
