package com.gic.enterprise.service.outer.impl;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

import com.gic.cloud.service.IndexDescApiService;
import com.gic.commons.util.CollectionUtil;
import com.gic.enterprise.constant.BillingAuditStatusEnum;
import com.gic.enterprise.constant.LicenseTypeEnum;
import com.gic.enterprise.dto.market.ServeLicenseDTO;
import com.gic.enterprise.error.Error;
import com.gic.enterprise.utils.LicenseUtils;
import com.gic.marketing.process.api.service.sms.SmsSendApiService;
import com.gic.member.tag.api.service.SceneCrowdGroupApiService;
import com.gic.open.api.service.EnterpriseLicenseApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.BusinessFrontResDTO;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.BusinessFrontResApiService;
import com.gic.auth.service.UserApiService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.constant.EnterpriseStatusTypeEnum;
import com.gic.enterprise.constants.Constants;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.dto.EnterpriseLicenseDTO;
import com.gic.enterprise.dto.EnterpriseListDTO;
import com.gic.enterprise.dto.EnterpriseResourceRelDTO;
import com.gic.enterprise.dto.market.*;
import com.gic.enterprise.entity.TabEnterprise;
import com.gic.enterprise.entity.TabEnterpriseInit;
import com.gic.enterprise.entity.TabEnterpriseResourceRel;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.BuyForGicQO;
import com.gic.enterprise.qo.EnterpriseQO;
import com.gic.enterprise.qo.OrderServiceQO;
import com.gic.enterprise.service.*;
import com.gic.open.api.dto.*;
import com.gic.open.api.dto.market.OrderBuyLicenseDTO;
import com.gic.open.api.dto.market.OrderCheckDTO;
import com.gic.open.api.service.AppTokenApiService;
import com.gic.open.api.service.OrderApiService;
import com.gic.open.api.service.ServeApiService;
import com.gic.open.api.service.market.MarketUserApiService;
import com.gic.open.api.service.market.OrderPurchaseApiService;
import com.gic.store.service.StoreGroupApiService;
import com.gic.store.utils.valid.ValidUtil;

/**
 * @author guojx
 * @date 2019/7/16 4:33 PM
 */
@Service("enterpriseApiService")
public class EnterpriseApiServiceImpl implements EnterpriseApiService {
    private static final Logger LOGGER = LogManager.getLogger(EnterpriseApiServiceImpl.class);
    @Autowired
    private EnterpriseService enterpriseService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private EnterpriseInitService enterpriseInitService;
    @Autowired
    private MarketUserApiService marketUserApiService;
    @Autowired
    private OrderPurchaseApiService orderPurchaseApiService;
    @Autowired
    private EnterpriseLicenseService enterpriseLicenseService;
    @Autowired
    private OrderApiService orderApiService;
    @Autowired
    private AppTokenApiService appTokenApiService;
    @Autowired
    private StoreGroupApiService storeGroupApiService;
    @Autowired
    private BillingAccountService billingAccountService;
    @Autowired
    private ServeApiService serveApiService;
    @Autowired
    private EnterpriseResourceRelService enterpriseResourceRelService;
    @Autowired
    private BusinessFrontResApiService businessFrontResApiService;
    @Autowired
    private PushMessageApiService pushMessageApiService;
    @Autowired
    private DataConfigApiService dataConfigApiService;
    @Autowired
    private SmsSendApiService smsSendApiService;
    @Autowired
    private EnterpriseLicenseApiService enterpriseLicenseApiService;
    @Autowired
    private IndexDescApiService indexDescApiService;
    @Autowired
    private SceneCrowdGroupApiService sceneCrowdGroupApiService;
    @Autowired
    private EnterpriseGoodsResourceApiService enterpriseGoodsResourceApiService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<Integer> saveEnterprise(EnterpriseDTO enterpriseDTO) {
        String validParam = validParam(enterpriseDTO, true);
        if (StringUtils.isNotBlank(validParam)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), validParam);
        }
        //新增商户信息
        int enterpriseId = enterpriseService.saveEnterprise(enterpriseDTO);

        //前置资源校验
        List<EnterpriseResourceRelDTO> resourceList = enterpriseDTO.getResourcesList();
        if (CollectionUtils.isEmpty(resourceList)) {
            ServiceResponse<List<BusinessFrontResDTO>> businessFrontResListResult = businessFrontResApiService
                    .listGroupResourceCode();
            if (businessFrontResListResult.isSuccess()) {
                List<BusinessFrontResDTO> businessFrontResList = businessFrontResListResult.getResult();
                if (CollectionUtils.isNotEmpty(businessFrontResList)) {
                    for (BusinessFrontResDTO res : businessFrontResList) {
                        //默认设置关闭数据
                        enterpriseResourceRelService.deleteResource(enterpriseId, res.getResourceCode());
                        EnterpriseResourceRelDTO enterpriseResourceRel = new EnterpriseResourceRelDTO();
                        enterpriseResourceRel.setEnterpriseId(enterpriseId);
                        enterpriseResourceRel.setResourceCode(res.getResourceCode());
                        enterpriseResourceRel.setResourceName(res.getResourceName());
                        enterpriseResourceRel.setResourceSwitch(0);
                        enterpriseResourceRelService.saveResource(enterpriseResourceRel);
                    }
                }
            }
        } else {
            LOGGER.info("新增商户-前置资源参数：{}", JSON.toJSONString(resourceList));
            saveBusinessResource(resourceList, enterpriseId);
        }

        //商户初始化配置信息生成
        enterpriseInitService.initEnterpriseConfigInfo(enterpriseId);

        //初始化计费中心账户
        billingAccountService.createBillingAccount(enterpriseId);

        //数据配置：会员消费配置
        dataConfigApiService.initMemberConsumeConfig(enterpriseId);
        //数据配置：实付配置默认数据
        dataConfigApiService.initActuallyPaidData(enterpriseId);
        //数据配置：单效取值配置默认数据
        dataConfigApiService.initSingleEffectConfig(enterpriseId);
        //业绩指标配置默认数据
        dataConfigApiService.initPerformanceConfig(enterpriseId);

        sceneCrowdGroupApiService.initSceneCrowdGroup(enterpriseId);

        //观云台字典初始化
        indexDescApiService.init(enterpriseId);

        UserDTO userDTO = EntityUtil.changeEntityNew(UserDTO.class, enterpriseDTO);
        //新增超级管理员
        userDTO.setEnterpriseId(enterpriseId);
        //超级管理员
        userDTO.setSuperAdmin(1);
        ServiceResponse userResult = userApiService.saveAdmin(userDTO);
        if (!userResult.isSuccess()) {
            //事物回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return userResult;
        }

        //初始化门店分组数据
        ServiceResponse<Integer> storeGroupInit = storeGroupApiService.insertDefaultStoreGroup(enterpriseId);
        if (!storeGroupInit.isSuccess()) {
            //事物回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return storeGroupInit;
        }

        return ServiceResponse.success(enterpriseId);
    }

    @Override
    public ServiceResponse<Integer> editEnterprise(EnterpriseDTO enterpriseDTO) {
        String validParam = validParam(enterpriseDTO, false);
        if (StringUtils.isNotBlank(validParam)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), validParam);
        }
        enterpriseService.editEnterprise(enterpriseDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<String> editResourceSwitch(Integer enterpriseId, String resources) {
        TabEnterprise tabEnterprise = enterpriseService.getEnterpriseById(enterpriseId);
        if (tabEnterprise == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID错误，查询不到数据");
        }
        LOGGER.info("编辑资源数据:{}", resources);
        List<EnterpriseResourceRelDTO> list = JSON.parseArray(resources, EnterpriseResourceRelDTO.class);
        saveBusinessResource(list, enterpriseId);
        return ServiceResponse.success(tabEnterprise.getEnterpriseName());
    }

    @Override
    public ServiceResponse<Map<String, String>> getResourceDict() {
        ServiceResponse<List<BusinessFrontResDTO>> businessFrontResListResult = businessFrontResApiService
                .listGroupResourceCode();
        Map<String, String> result = new HashMap<>(16);
        if (businessFrontResListResult.isSuccess()) {
            List<BusinessFrontResDTO> businessFrontResList = businessFrontResListResult.getResult();
            if (CollectionUtils.isNotEmpty(businessFrontResList)) {
                for (BusinessFrontResDTO res : businessFrontResList) {
                    result.put(res.getResourceCode(), res.getResourceName());
                }
            }
        }
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<Integer> editEnterpriseInfo(EnterpriseDTO enterpriseDTO) {
        TabEnterprise tabEnterprise = enterpriseService.getEnterpriseById(enterpriseDTO.getEnterpriseId());
        if (tabEnterprise == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "企业ID错误，查询不到数据");
        }
        if (StringUtils.isNotBlank(enterpriseDTO.getEnterpriseName())) {
            if (enterpriseService.isRepeatEnterpriseName(enterpriseDTO.getEnterpriseName(),
                    enterpriseDTO.getEnterpriseId())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户名称不能重复");
            }
        }
        if (StringUtils.isNotBlank(enterpriseDTO.getEnterpriseName())) {
            if (enterpriseService.isRepeatCompanyName(enterpriseDTO.getCompanyName(),
                    enterpriseDTO.getEnterpriseId())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "公司名称不能重复");
            }
        }
        enterpriseService.editEnterprise(enterpriseDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<String> modifyEnterpriseLogo(Integer enterpriseId, String logoUrl) {
        TabEnterprise tabEnterprise = enterpriseService.getEnterpriseById(enterpriseId);
        if (tabEnterprise == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "企业ID错误，查询不到数据");
        }
        EnterpriseDTO enterpriseDTO = new EnterpriseDTO();
        enterpriseDTO.setEnterpriseId(enterpriseId);
        enterpriseDTO.setLogo(logoUrl);
        enterpriseService.editEnterprise(enterpriseDTO);
        return ServiceResponse.success(tabEnterprise.getEnterpriseName());
    }

    @Override
    public ServiceResponse<EnterpriseDTO> getEnterpriseById(Integer enterpriseId) {
        TabEnterprise tabEnterprise = enterpriseService.getEnterpriseById(enterpriseId);
        if (tabEnterprise == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID输入有误");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(EnterpriseDTO.class, tabEnterprise));
    }

    @Override
    public ServiceResponse<List<EnterpriseResourceRelDTO>> getBusinessFrontRes(Integer enterpriseId) {
        List<TabEnterpriseResourceRel> list = enterpriseResourceRelService.getByEnterpriseId(enterpriseId);
        if (CollectionUtils.isNotEmpty(list)) {
            return ServiceResponse.success(EntityUtil.changeEntityListNew(EnterpriseResourceRelDTO.class, list));
        } else {
            ServiceResponse<List<BusinessFrontResDTO>> businessFrontResListResult = businessFrontResApiService
                    .listGroupResourceCode();
            if (businessFrontResListResult.isSuccess()) {
                List<BusinessFrontResDTO> resourceList = businessFrontResListResult.getResult();
                if (CollectionUtils.isNotEmpty(resourceList)) {
                    List<EnterpriseResourceRelDTO> result = new ArrayList<>(resourceList.size());
                    for (BusinessFrontResDTO res : resourceList) {
                        EnterpriseResourceRelDTO temp = new EnterpriseResourceRelDTO();
                        temp.setResourceSwitch(0);
                        temp.setResourceName(res.getResourceName());
                        temp.setResourceCode(res.getResourceCode());
                        result.add(temp);
                    }
                    return ServiceResponse.success(result);
                }
            }
            return ServiceResponse.success(Collections.emptyList());
        }
    }

    @Override
    public ServiceResponse disableEnterprise(Integer enterpriseId) {
        TabEnterprise tabEnterprise = enterpriseService.getEnterpriseById(enterpriseId);
        if (tabEnterprise == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID输入有误");
        }
        if (tabEnterprise.getServiceStatus().intValue() != EnterpriseStatusTypeEnum.EFFECTIVE.getCode()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "服务不是正常状态，不能停用");
        }
        enterpriseService.changeEnterpriseStatus(enterpriseId, EnterpriseStatusTypeEnum.DISABLE.getCode());
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse enableEnterprise(Integer enterpriseId) {
        TabEnterprise tabEnterprise = enterpriseService.getEnterpriseById(enterpriseId);
        if (tabEnterprise == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID输入有误");
        }
        if (tabEnterprise.getServiceStatus().intValue() != EnterpriseStatusTypeEnum.DISABLE.getCode()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "服务不是停用状态，不能启用");
        }
        enterpriseService.changeEnterpriseStatus(enterpriseId, EnterpriseStatusTypeEnum.EFFECTIVE.getCode());
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<EnterpriseListDTO>> listEnterprise(String search, Integer serviceVersion,
            Integer status, String orderBy, Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page page = enterpriseService.listEnterprise(search, serviceVersion, status, orderBy,
                pageNum, pageSize);
        Page<EnterpriseListDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<Page<EnterpriseListDTO>> listEnterprise(EnterpriseQO params) {
        com.github.pagehelper.Page page = enterpriseService.listEnterprise(params);
        Page<EnterpriseListDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<List<EnterpriseDTO>> listEnterpriseByName(String search) {
        return ServiceResponse.success(enterpriseService.listEnterpriseByName(search));
    }

    @Override
    public ServiceResponse<List<EnterpriseDTO>> listEnterpriseBySearch(String search) {
        return ServiceResponse.success(enterpriseService.listEnterpriseBySearch(search));
    }

    @Override
    public ServiceResponse initEnterprise(Integer enterpriseId) {
        TabEnterprise tabEnterprise = enterpriseService.getEnterpriseById(enterpriseId);
        if (tabEnterprise == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID输入有误");
        }
        //验证是否可以初始化
        List<TabEnterpriseInit> listByParentId = enterpriseInitService.listByParentId(null, enterpriseId);
        if (CollectionUtils.isNotEmpty(listByParentId)) {
            for (TabEnterpriseInit init : listByParentId) {
                if (init.getIsInitConfig().intValue() == 0) {
                    return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(),
                            "请" + init.getConfigName() + "的初始化配置");
                }
            }
        }
        //初始化操作
        enterpriseService.initEnterprise(enterpriseId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<EnterpriseDTO>> listEnterpriseByIds(List<Integer> enterpriseIdList) {
        List<TabEnterprise> list = this.enterpriseService.listEnterpriseByIds(enterpriseIdList);
        if (CollectionUtils.isNotEmpty(list)) {
            return ServiceResponse.success(EntityUtil.changeEntityListByJSON(EnterpriseDTO.class, list));
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<EnterpriseDTO> getEnterpriseByName(String enterpriseName) {
        TabEnterprise enterpriseByName = this.enterpriseService.getEnterpriseByName(enterpriseName);
        return ServiceResponse.success(EntityUtil.changeEntityByJSON(EnterpriseDTO.class, enterpriseByName));
    }

    @Override
    public ServiceResponse<List<EnterpriseDTO>> listEnterpriseByPhoneNumber(String phoneNumber) {
        ServiceResponse<List<UserDTO>> listServiceResponse = this.userApiService.listUserByPhoneNumber(phoneNumber,
                Constants.NATION_CODE);
        if (listServiceResponse.isSuccess()) {
            List<UserDTO> result = listServiceResponse.getResult();
            if (CollectionUtils.isEmpty(result)) {
                return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "用户不存在");
            } else {
                List<Integer> idList = new ArrayList<>();
                for (UserDTO userDTO : result) {
                    idList.add(userDTO.getEnterpriseId());
                }
                ServiceResponse<List<EnterpriseDTO>> serviceResponse = this.listEnterpriseByIds(idList);
                if (serviceResponse.isSuccess()) {
                    List<EnterpriseDTO> list = serviceResponse.getResult();
                    if (CollectionUtils.isEmpty(list)) {
                        return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "用户不存在");
                    } else {
                        return ServiceResponse.success(list);
                    }
                }
            }
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<MarketServiceDTO> buyForGicForNew(BuyForGicQO basicParams) {
        MarketServiceDTO marketServiceDTO = new MarketServiceDTO();

        //是否有商品管理权限
        boolean hasGoodsAuth = enterpriseGoodsResourceApiService.validGoodsAuth(basicParams.getEnterpriseId().intValue()).getResult();

        List<com.gic.enterprise.dto.market.OrderBuyLicenseDTO> orderBuyLicenseDTOList = basicParams.getLicenseList();
        List<OrderBuyLicenseDTO> licenseListParams = EntityUtil.changeEntityListNew(OrderBuyLicenseDTO.class,
                orderBuyLicenseDTOList);

        //是否是服务开通操作
        boolean isOpenService = basicParams.getOrderType() != null && basicParams.getOrderType() == 1;
        if (isOpenService && !hasGoodsAuth) {
            //没有商品权限
            Long goodsLicenseId = getGoodsLicenseId(basicParams.getEnterpriseId(), basicParams.getOrderType());
            if (goodsLicenseId != null) {
                //有商品license，给一个默认的商品license 9个
                OrderBuyLicenseDTO orderBuyLicenseDTO = new OrderBuyLicenseDTO();
                orderBuyLicenseDTO.setNum(9);
                orderBuyLicenseDTO.setLicenseId(goodsLicenseId);
                licenseListParams.add(orderBuyLicenseDTO);
            }
        }


        ServiceResponse<OrderCheckDTO> result = orderPurchaseApiService.buyForGic(basicParams.getEnterpriseId(),
                basicParams.getName(), basicParams.getSkuId(), basicParams.getCouponLogId(), basicParams.getOrderType(),
                licenseListParams, basicParams.getPrice(), basicParams.isCommitFlag(), null);
        if (result.isSuccess()) {
            OrderCheckDTO orderClerk = result.getResult();
            OrderDTO orderDTO = orderClerk.getOrderDTO();
            if (orderClerk.getCanBuy().intValue() == 0) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), orderClerk.getReason());
            }
            List<OrderItemDTO> orderItemList = orderDTO.getOrderItemList();
            BasicServiceDTO basicServiceDTO = new BasicServiceDTO();
            List<LicenseDTO> licenseList = new ArrayList<>(orderItemList.size());
            List<ServeSkuDTO> serveSkuList = orderClerk.getSkuList();
            List<SkuDTO> skuList = new ArrayList<>(serveSkuList.size());
            for (ServeSkuDTO serveSkuDTO : serveSkuList) {
                SkuDTO skuDTO = EntityUtil.changeEntityNew(SkuDTO.class, serveSkuDTO);
                skuDTO.setSkuId(long2String(serveSkuDTO.getSkuId()));
                skuList.add(skuDTO);
            }
            for (OrderItemDTO orderItem : orderItemList) {
                Integer serviceType = orderItem.getServeType();
                if (serviceType == 1) {
                    //服务
                    basicServiceDTO = EntityUtil.changeEntityNew(BasicServiceDTO.class, orderItem);
                    basicServiceDTO.setSkuId(long2String(orderItem.getSkuId()));
                    basicServiceDTO.setServeId(long2String(orderItem.getServeId()));
                } else if (serviceType == 2) {
                    //license
                    LicenseDTO license = EntityUtil.changeEntityNew(LicenseDTO.class, orderItem);
                    license.setLicenseDetail(EntityUtil.changeEntityNew(ServeLicenseDTO.class, orderItem.getLicenseDetail()));
                    //没有商品权限
                    if (!hasGoodsAuth) {
                        if (LicenseUtils.isGoodsLicense(orderItem.getLicenseDetail().getLicenseCode())) {
                            continue;
                        }
                    }
                    license.setSkuId(long2String(orderItem.getSkuId()));
                    license.setServeId(long2String(orderItem.getServeId()));
                    licenseList.add(license);
                }
            }
            marketServiceDTO.setBasicService(basicServiceDTO);
            marketServiceDTO.setLicenseList(licenseList);
            marketServiceDTO.setCouponId(long2String(orderDTO.getCouponId()));
            marketServiceDTO.setCouponCheckAmount(orderDTO.getCouponCheckAmount());
            marketServiceDTO.setEndDate(orderClerk.getEndDate());
            marketServiceDTO.setSkuList(skuList);
            marketServiceDTO.setCanBuy(orderClerk.getCanBuy());
            marketServiceDTO.setPrice(orderDTO.getPayAmount());
            marketServiceDTO.setVerifyCode(orderDTO.getVerifyCode());
            marketServiceDTO.setIsvId(long2String(orderDTO.getIsvId()));
            marketServiceDTO.setAppServeId(long2String(orderClerk.getAppServeId()));
            LOGGER.info("基础服务信息:{}", JSON.toJSONString(marketServiceDTO));

            return ServiceResponse.success(marketServiceDTO);
        }
        return ServiceResponse.failure(result.getCode(), result.getMessage());
    }

    @Override
    public ServiceResponse<List<EnterpriseLicenseDTO>> listEnterpriseLicense(Integer enterpriseId) {
        ServiceResponse<List<ApplicationLicenseDTO>> hasBuyLicense = enterpriseLicenseApiService.listEnterpriseBuyLicense(Arrays.asList(enterpriseId));
        LOGGER.info("商户ID：{}，license信息:{}", enterpriseId, JSON.toJSONString(hasBuyLicense));
        if (hasBuyLicense.isSuccess()) {
            List<ApplicationLicenseDTO> licenseList = hasBuyLicense.getResult();
            if (CollectionUtils.isNotEmpty(licenseList)) {
                return ServiceResponse.success(licenseList.stream()
                .map(e -> new EnterpriseLicenseDTO()
                            .setLicenseId(e.getLicenseId())
                            .setUpperLimit(e.getQuantity())
                            .setLicenseName(e.getLicenseName())
                            .setLicenseCode(e.getLicenseCode())
                ).collect(Collectors.toList()));
            }
        }
        return ServiceResponse.success(Collections.emptyList());
    }

    @Override
    public ServiceResponse<EnterpriseLicenseDTO> getEnterpriseLicense(Integer enterpriseId, String licenseCode) {
        ServiceResponse<List<EnterpriseLicenseDTO>> result = listEnterpriseLicense(enterpriseId);
        if (result.isSuccess()) {
            List<EnterpriseLicenseDTO> list = result.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                return ServiceResponse.success(list.stream().filter(e -> e.getLicenseCode().equals(licenseCode)).collect(Collectors.toList()).get(0));
            }
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<OrderGicDTO>> listOrder(OrderServiceQO params) {
        Page<OrderGicDTO> resultPage = new Page<>();

        Page<Object> page = new Page<>();
        page.setCurrentPage(params.getCurrentPage());
        page.setPageSize(params.getPageSize());
        ServiceResponse<Page<OrderGicHbDTO>> list = orderApiService.listOrderForGic(page, params.getOrderNum(),
                params.getEnterpriseId(), params.getOperName(), params.getOrderType(), params.getStatus());

        if (list.isSuccess()) {
            Page<OrderGicHbDTO> orderGicHbPage = list.getResult();
            resultPage = EntityUtil.changeEntityNew(Page.class, orderGicHbPage);
            List<OrderGicDTO> orderGicDTOList = EntityUtil.changeEntityListNew(OrderGicDTO.class, orderGicHbPage.getResult());
            resultPage.setResult(orderGicDTOList);
        }
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<Void> cancelOrder(Long orderId) {
        return orderPurchaseApiService.closeOrder(null, orderId, "手工关闭订单");
    }

    @Override
    public ServiceResponse<Void> buyServiceSuccess(Integer enterpriseId, String json) {
        TabEnterprise tabEnterprise = enterpriseService.getEnterpriseById(enterpriseId);
        if (tabEnterprise == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID错误，查询不到数据");
        }
        ApplicationTokenDTO serviceDTO = JSON.parseObject(json, ApplicationTokenDTO.class);
        List<ApplicationLicenseDTO> licenseList = serviceDTO.getLicenseList();
        List<com.gic.enterprise.dto.market.OrderBuyLicenseDTO> orderBuyLicenseDTOList = new ArrayList<>();
        //用来存储
        String versionCode = serviceDTO.getVersionCode();
        for (ApplicationLicenseDTO license : licenseList) {
            //license
            com.gic.enterprise.dto.market.OrderBuyLicenseDTO licenseTemp = new com.gic.enterprise.dto.market.OrderBuyLicenseDTO();
            licenseTemp.setLicenseId(license.getLicenseId());
            licenseTemp.setLicenseName(license.getLicenseName());
            licenseTemp.setNum(license.getQuantity());
            orderBuyLicenseDTOList.add(licenseTemp);
        }

        tabEnterprise.setVersionCode(versionCode);
        tabEnterprise.setAuditTime(serviceDTO.getExpireTime());
        tabEnterprise.setVersionName(serviceDTO.getVersionName());
        tabEnterprise.setServiceStatus(EnterpriseStatusTypeEnum.EFFECTIVE.getCode());
        enterpriseService.editEnterprise(EntityUtil.changeEntityNew(EnterpriseDTO.class, tabEnterprise));
        // license添加
        List<EnterpriseLicenseDTO> enterpriseLicenseList = new ArrayList<>();
        Date now = new Date();
        for (com.gic.enterprise.dto.market.OrderBuyLicenseDTO orderBuyLicenseDTO : orderBuyLicenseDTOList) {
            EnterpriseLicenseDTO enterpriseLicenseDTO = new EnterpriseLicenseDTO();
            enterpriseLicenseDTO.setCreateTime(now);
            enterpriseLicenseDTO.setUpdateTime(now);
            enterpriseLicenseDTO.setLicenseId(orderBuyLicenseDTO.getLicenseId());
            enterpriseLicenseDTO.setUpperLimit(orderBuyLicenseDTO.getNum());
            enterpriseLicenseDTO.setLicenseName(orderBuyLicenseDTO.getLicenseName());
            enterpriseLicenseDTO.setEnterpriseId(enterpriseId);
            enterpriseLicenseList.add(enterpriseLicenseDTO);
        }
        enterpriseLicenseService.deleteByEnterpriseId(enterpriseId);
        enterpriseLicenseService.saveFetch(enterpriseLicenseList);

        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> overdueReminder(String param) {
        List<EnterpriseDTO> list = this.enterpriseService.listEnterprise();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Date now = new Date();
        for(EnterpriseDTO enterpriseDTO : list){
            try{
                Date auditTime = enterpriseDTO.getAuditTime();
                long days = (sdf.parse(sdf.format(auditTime)).getTime() - sdf.parse(sdf.format(now)).getTime())/(1000 * 60 * 60 * 24);
                if(days <= 7){
                    this.pushMessageApiService.pushMessage(enterpriseDTO.getEnterpriseId(), "系统消息",
                            "GIC版本即将过期", "您的GIC版本即将在"+ days +"天到期，为了不影响业务的开展，请及时续费");

                    //发送超级管理员短信通知
                    String[] smsArr = new String[]{enterpriseDTO.getEnterpriseName()};
                    ServiceResponse<UserDTO> userResult = userApiService.getUserByEnterpriseId(enterpriseDTO.getEnterpriseId());
                    UserDTO userDTO = userResult.getResult();
                    ServiceResponse<Void> smsSendResult = smsSendApiService.sendPlatformSms("GICJFZX002",
                            userDTO.getEnterpriseId(), userDTO.getPhoneAreaCode(), userDTO.getPhoneNumber(), smsArr);
                    if (!smsSendResult.isSuccess()) {
                        LOGGER.warn(smsSendResult.getMessage());
                    }
                }
            }catch (Exception e){
                LOGGER.warn("exception", e);
            }
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> expireReminder(String param) {
        List<EnterpriseDTO> list = this.enterpriseService.listEnterprise();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String today = sdf.format(new Date());
        for (EnterpriseDTO enterpriseDTO : list) {
            if (StringUtils.isNotBlank(enterpriseDTO.getVersionCode())) {
                Date expireDate = enterpriseDTO.getAuditTime();
                if (today.compareTo(sdf.format(expireDate)) >= 0) {
                    //过期时间小于等于今天，发短信
                    //发送超级管理员短信通知
                    String[] smsArr = new String[]{enterpriseDTO.getEnterpriseName()};
                    ServiceResponse<UserDTO> userResult = userApiService.getUserByEnterpriseId(enterpriseDTO.getEnterpriseId());
                    UserDTO userDTO = userResult.getResult();
                    ServiceResponse<Void> smsSendResult = smsSendApiService.sendPlatformSms("GICJFZX003",
                            userDTO.getEnterpriseId(), userDTO.getPhoneAreaCode(), userDTO.getPhoneNumber(), smsArr);
                    if (!smsSendResult.isSuccess()) {
                        LOGGER.warn(smsSendResult.getMessage());
                    }
                }
            }
        }
        return ServiceResponse.success();
    }

    private String getVersionNameFromJson(String jsonStr) {
        JSONObject jsonObject = JSONObject.parseObject(jsonStr);
        JSONObject version = jsonObject.getJSONObject("version");
        return version.getString("name");
    }

    private static String long2String(Long id) {
        if (id == null) {
            return null;
        }
        return id.toString();
    }

    private Long getGoodsLicenseId(Long enterpriseId, Integer orderType) {
        ServiceResponse<OrderCheckDTO> result = orderPurchaseApiService.buyForGic(enterpriseId,
                null, null,null, orderType,
                null, null,false, null);
        if (result.isSuccess()) {
            OrderCheckDTO orderClerk = result.getResult();
            OrderDTO orderDTO = orderClerk.getOrderDTO();
            if (orderClerk.getCanBuy().intValue() == 0) {
                return null;
            }
            List<OrderItemDTO> orderItemList = orderDTO.getOrderItemList();
            for (OrderItemDTO orderItem : orderItemList) {
                Integer serviceType = orderItem.getServeType();
                if (serviceType == 2) {
                    //license
                    if (LicenseUtils.isGoodsLicense(orderItem.getLicenseDetail().getLicenseCode())) {
                        return orderItem.getServeId();
                    }
                }
            }
        }
        return null;
    }

    /**
     * 覆盖商户前置资源
     * @param list
     * @param enterpriseId
     */
    private void saveBusinessResource(List<EnterpriseResourceRelDTO> list, Integer enterpriseId) {
        for (EnterpriseResourceRelDTO res : list) {
            enterpriseResourceRelService.deleteResource(enterpriseId, res.getResourceCode());
            EnterpriseResourceRelDTO enterpriseResourceRel = new EnterpriseResourceRelDTO();
            enterpriseResourceRel.setEnterpriseId(enterpriseId);
            enterpriseResourceRel.setResourceCode(res.getResourceCode());
            enterpriseResourceRel.setResourceName(res.getResourceName());
            enterpriseResourceRel.setResourceSwitch(res.getResourceSwitch());
            enterpriseResourceRelService.saveResource(enterpriseResourceRel);
        }
    }

    /**
     * 新增/编辑参数校验
     * @param enterpriseDTO
     * @param isAdd
     * @return null表示正确
     */
    private String validParam(EnterpriseDTO enterpriseDTO, boolean isAdd) {
        Integer enterpriseId = enterpriseDTO.getEnterpriseId();
        TabEnterprise tabEnterprise = enterpriseService.getEnterpriseById(enterpriseId);
        ServiceResponse paramResult;
        //新增情况
        if (isAdd) {
            //为了分库指定主键ID
            if (enterpriseId == null) {
                return "商户ID不能为空";
            }
            if (tabEnterprise != null) {
                return "商户ID已存在，请更换";
            }
            paramResult = ValidUtil.allCheckValidate(enterpriseDTO,
                    EnterpriseDTO.SaveEnterpriseValid.class);

        } else {
            //valid param
            paramResult = ValidUtil.allCheckValidate(enterpriseDTO, EnterpriseDTO.BasicInfoEditValid.class,
                    EnterpriseDTO.EditEnterpriseValid.class);
            if (tabEnterprise == null) {
                return "商户ID错误，查询不到数据";
            }
        }
        if (!paramResult.isSuccess()) {
            return paramResult.getMessage();
        }
        if (enterpriseService.isRepeatEnterpriseName(enterpriseDTO.getEnterpriseName(), enterpriseId)) {
            return "商户名称不能重复";
        }
        if (enterpriseService.isRepeatCompanyName(enterpriseDTO.getCompanyName(), enterpriseId)) {
            return "公司名称不能重复";
        }
        return null;
    }

}
