package com.gic.enterprise.service.outer.impl.union;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.UserApiService;
import com.gic.enterprise.constant.union.UnionEnterpriseAuthDetailStatusEnum;
import com.gic.enterprise.constant.union.UnionEnterpriseAuthResOperateEnum;
import com.gic.enterprise.constant.union.UnionEnterpriseAuthResTypeEnum;
import com.gic.enterprise.constant.union.UnionEnterpriseAuthStatusEnum;
import com.gic.enterprise.dto.union.UnionEnterpriseAuthDTO;
import com.gic.enterprise.entity.TabEnterprise;
import com.gic.enterprise.entity.TabSysUnionEnterpriseAuth;
import com.gic.enterprise.entity.TabSysUnionEnterpriseAuthResDetail;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.exception.CommonException;
import com.gic.enterprise.service.*;
import com.gic.marketing.process.api.service.sms.SmsSendApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 联合资源回调接口
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/2/8 10:27 AM 
 */
@Service("unionEnterpriseAuthResDetailApiService")
public class UnionEnterpriseAuthResDetailApiServiceImpl implements UnionEnterpriseAuthResDetailApiService{
    private static Logger LOGGER = LogManager.getLogger(UnionEnterpriseAuthResDetailApiServiceImpl.class);
    @Autowired
    private UnionEnterpriseAuthResDetailService unionEnterpriseAuthResDetailService;
    @Autowired
    private UnionEnterpriseAuthService unionEnterpriseAuthService;
    @Autowired
    private UnionEnterpriseAuthResLogService unionEnterpriseAuthResLogService;
    @Autowired
    private SmsSendApiService smsSendApiService;
    @Autowired
    private EnterpriseService enterpriseService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private PushMessageApiService pushMessageApiService;

    private static String SMS_CODE_TO_OWN = "GICSHHT002";
    private static String SMS_CODE_TO_UNION = "GICSHHT012";
    private static String SMS_CODE_TO_RELIEVE = "GICSHHT013";

    @Override
    public ServiceResponse<Void> authDataCallback() {
        return null;
    }

    @Override
    public ServiceResponse<Void> unionForStore(@NotNull Integer authStatus, @NotNull Integer ownEnterpriseId, @NotNull Integer unionEnterpriseId,
                                               String authKey, String remark) {
        unionCommon(UnionEnterpriseAuthResTypeEnum.STORE.getCode(), authStatus, ownEnterpriseId, unionEnterpriseId, authKey, remark);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> relieveForStore(@NotNull Integer authStatus, @NotNull Integer ownEnterpriseId, @NotNull Integer unionEnterpriseId) {
        relieveCommon(UnionEnterpriseAuthResTypeEnum.STORE.getCode(), authStatus, ownEnterpriseId, unionEnterpriseId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> unionForUser(@NotNull Integer authStatus, @NotNull Integer ownEnterpriseId, @NotNull Integer unionEnterpriseId, String authKey, String remark) {
        unionCommon(UnionEnterpriseAuthResTypeEnum.USER_AREA.getCode(), authStatus, ownEnterpriseId, unionEnterpriseId, authKey, remark);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> relieveForUser(@NotNull Integer authStatus, @NotNull Integer ownEnterpriseId, @NotNull Integer unionEnterpriseId) {
        relieveCommon(UnionEnterpriseAuthResTypeEnum.USER_AREA.getCode(), authStatus, ownEnterpriseId, unionEnterpriseId);
        return ServiceResponse.success();
    }

    private void relieveCommon(Integer resourceType, Integer authStatus, Integer ownEnterpriseId, Integer unionEnterpriseId) {
        TabSysUnionEnterpriseAuth old = unionEnterpriseAuthService.getUnionEnterprise(ownEnterpriseId, unionEnterpriseId);
        if (old == null) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "联合商户不存在");
        }
        TabSysUnionEnterpriseAuthResDetail detail = unionEnterpriseAuthResDetailService
                .getUnionEnterpriseAuthResDetail(old.getUnionEnterpriseAuthId(), resourceType);
        boolean isAllRelieveSuccess = UnionEnterpriseAuthDetailStatusEnum.HAS_RELIEVE.getCode().equals(authStatus);
        if (isAllRelieveSuccess) {
            unionEnterpriseAuthResLogService.saveUnionEnterpriseAuthResLog(old.getUnionEnterpriseAuthId(), resourceType,
                    UnionEnterpriseAuthResOperateEnum.RELIEVED.getMessage(), null, new Date());
        } else {
            unionEnterpriseAuthResLogService.saveUnionEnterpriseAuthResLog(old.getUnionEnterpriseAuthId(), resourceType,
                    UnionEnterpriseAuthResOperateEnum.RELIEVED.getMessage(), new Date(), null);
        }
        //如果是，则判断其他资源是否已经完成
        UnionEnterpriseAuthDTO dto = new UnionEnterpriseAuthDTO();
        dto.setUnionEnterpriseAuthId(old.getUnionEnterpriseAuthId());

        if (isAllRelieveSuccess && isAllDealSuccess(old.getUnionEnterpriseAuthId(), resourceType, false)) {
            //更新最新联合时间（）
            dto.setLastRelieveTime(new Date());
            dto.setStatusFlag(UnionEnterpriseAuthStatusEnum.RELIEVED.getCode());
        } else {
            dto.setStatusFlag(UnionEnterpriseAuthStatusEnum.RELIEVING.getCode());
        }

        //如果是解除失败，则要发送短信
        if (UnionEnterpriseAuthDetailStatusEnum.RELIEVE_FAIL.getCode().equals(authStatus)) {
            Map<String, String> enterpriseMap = getEnterpriseNameMap(ownEnterpriseId, unionEnterpriseId);
            //发送短信
            String[] smsArrOfOwn = new String[] { enterpriseMap.get(unionEnterpriseId), UnionEnterpriseAuthResTypeEnum.getMessage(resourceType)};
            ServiceResponse<UserDTO> userResponse = userApiService.getUserByEnterpriseId(ownEnterpriseId);
            //发送手机验证码
            ServiceResponse<Void> smsSendResult = smsSendApiService.sendPlatformSms(SMS_CODE_TO_RELIEVE,
                    ownEnterpriseId,
                    userResponse.getResult().getPhoneAreaCode(),
                    userResponse.getResult().getPhoneNumber(),
                    smsArrOfOwn);
            if (!smsSendResult.isSuccess()) {
                LOGGER.warn("资源解除失败发送短信错误信息:{}", JSON.toJSONString(smsSendResult));
            }
            pushMessageApiService.pushUserMessage(ownEnterpriseId, userResponse.getResult().getUserId(), "系统消息",
                    "解除联合商户",
                    "抱歉，解除联合过程中出现异常，收回" + UnionEnterpriseAuthResTypeEnum.getMessage(resourceType) + "出现异常，请联系达摩客服解决");
        }

        unionEnterpriseAuthService.update(dto);
        unionEnterpriseAuthResDetailService.union(detail.getUnionEnterpriseAuthResDetailId(), authStatus, null, null);
    }

    private void unionCommon(Integer resourceType, Integer authStatus, Integer ownEnterpriseId, Integer unionEnterpriseId, String authKey, String remark) {
        TabSysUnionEnterpriseAuth old = unionEnterpriseAuthService.getUnionEnterprise(ownEnterpriseId, unionEnterpriseId);
        if (old == null) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "联合商户不存在");
        }
        TabSysUnionEnterpriseAuthResDetail detail = unionEnterpriseAuthResDetailService
                .getUnionEnterpriseAuthResDetail(old.getUnionEnterpriseAuthId(), resourceType);

        boolean isUnionSuccess = UnionEnterpriseAuthDetailStatusEnum.HAS_AUTH.getCode().equals(authStatus);
        //日志
        if (isUnionSuccess) {
            unionEnterpriseAuthResLogService.saveUnionEnterpriseAuthResLog(old.getUnionEnterpriseAuthId(), resourceType,
                    UnionEnterpriseAuthResOperateEnum.UNION.getMessage(), null, new Date());
        } else {
            unionEnterpriseAuthResLogService.saveUnionEnterpriseAuthResLog(old.getUnionEnterpriseAuthId(), resourceType,
                    UnionEnterpriseAuthResOperateEnum.UNION.getMessage(), new Date(), null);
        }
        //如果是，则判断其他资源是否已经完成
        if (isUnionSuccess && isAllDealSuccess(old.getUnionEnterpriseAuthId(), resourceType, true)) {
            //更新最新联合时间（）
            UnionEnterpriseAuthDTO dto = new UnionEnterpriseAuthDTO();
            dto.setUnionEnterpriseAuthId(old.getUnionEnterpriseAuthId());
            dto.setLastUnionTime(new Date());
            unionEnterpriseAuthService.update(dto);

            Map<String, String> enterpriseMap = getEnterpriseNameMap(ownEnterpriseId, unionEnterpriseId);
            sendSms(ownEnterpriseId, enterpriseMap.get(ownEnterpriseId), enterpriseMap.get(unionEnterpriseId), true);
            sendSms(unionEnterpriseId, enterpriseMap.get(ownEnterpriseId), enterpriseMap.get(unionEnterpriseId), false);
        }
        unionEnterpriseAuthResDetailService.union(detail.getUnionEnterpriseAuthResDetailId(), authStatus, authKey, remark);
    }

    private void sendSms(Integer enterpriseId, String ownEnterpriseName, String unionEnterpriseName, boolean sendSmsToOwn) {
        //发送短信
        String[] smsArrOfOwn = new String[] { sendSmsToOwn ? ownEnterpriseName : unionEnterpriseName,
                sendSmsToOwn ? unionEnterpriseName : ownEnterpriseName};
        ServiceResponse<UserDTO> userResponse = userApiService.getUserByEnterpriseId(enterpriseId);
        //发送手机验证码
        ServiceResponse<Void> smsSendResult = smsSendApiService.sendPlatformSms(sendSmsToOwn ? SMS_CODE_TO_OWN : SMS_CODE_TO_UNION,
                enterpriseId,
                userResponse.getResult().getPhoneAreaCode(),
                userResponse.getResult().getPhoneNumber(),
                smsArrOfOwn);
        if (!smsSendResult.isSuccess()) {
            LOGGER.warn("全部联合成功发送短信错误信息:{}", JSON.toJSONString(smsSendResult));
        }
    }

    /**
     * 其他资源是否授权成功/解除成功
     * @param unionEnterpriseAuthId
     * @param resourceType
     * @param isUnion
     * @return
     */
    private boolean isAllDealSuccess(Integer unionEnterpriseAuthId, Integer resourceType, boolean isUnion) {
        Integer authStatus = UnionEnterpriseAuthDetailStatusEnum.HAS_AUTH.getCode();
        if (!isUnion) {
            authStatus = UnionEnterpriseAuthDetailStatusEnum.HAS_RELIEVE.getCode();
        }
        boolean isAllDealSuccess = true;
        List<TabSysUnionEnterpriseAuthResDetail> resList = unionEnterpriseAuthResDetailService.listUnionEnterpriseAuthResDetail(unionEnterpriseAuthId, null);
        if (CollectionUtils.isNotEmpty(resList)) {
            for (TabSysUnionEnterpriseAuthResDetail temp : resList) {
                if (!temp.getResourceType().equals(resourceType) && !temp.getAuthStatus().equals(authStatus)) {
                    isAllDealSuccess = false;
                }
            }
        }
        return isAllDealSuccess;
    }

    private Map<String, String> getEnterpriseNameMap(Integer ownEnterpriseId, Integer unionEnterpriseId) {
        List<Integer> enterpriseIdList = new ArrayList<>(2);
        enterpriseIdList.add(ownEnterpriseId);
        enterpriseIdList.add(unionEnterpriseId);
        List<TabEnterprise> enterpriseList = enterpriseService.listEnterpriseByIds(enterpriseIdList);
        Map<String, String> enterpriseMap = enterpriseList.stream().collect(Collectors.toMap(e -> e.getEnterpriseId().toString(), e -> e.getEnterpriseName()));
        return enterpriseMap;
    }

}
