package com.gic.enterprise.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.AuditorDTO;
import com.gic.auth.service.AuditorApiService;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.ano.IgnoreLogin;
import com.gic.enterprise.dto.AuditLogDTO;
import com.gic.enterprise.dto.ProjectItemDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.AuditLogQO;
import com.gic.enterprise.service.AuditLogApiService;
import com.gic.enterprise.service.ProjectItemApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetail;
import com.gic.enterprise.utils.UserDetailUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-10-30 1:21
 */
@RestController
public class AuditLogController {

    @Autowired
    private AuditLogApiService auditLogApiService;
    @Autowired
    private AuditorApiService auditorApiService;
    @Autowired
    private ProjectItemApiService projectItemApiService;

    @RequestMapping("/audit")
    @IgnoreLogin
    public RestResponse audit(@Validated(AuditLogQO.AuditView.class) AuditLogQO auditLogQO) {
        ServiceResponse<AuditLogDTO> auditorLogRes = auditLogApiService.getAuditLog(auditLogQO.getAuditLogId());
        if (!auditorLogRes.isSuccess()) {
            return RestResponse.failure(auditorLogRes.getCode(), auditorLogRes.getMessage());
        }
        AuditLogDTO oldAuditLog = auditorLogRes.getResult();

        AuditLogDTO auditLogDTO = new AuditLogDTO();
        auditLogDTO.setAuditLogId(auditLogQO.getAuditLogId());
        auditLogDTO.setAuditResult(auditLogQO.getAuditResult());
        auditLogDTO.setAuditReason(auditLogQO.getAuditReason());

        UserDetail userDetail = UserDetailUtils.getUserDetail();
        AuditorDTO auditorDTO;
        if (StringUtils.isNotBlank(auditLogQO.getOpenid())) {
            //微信端审核
            ServiceResponse<AuditorDTO> auditorServiceResponse = auditorApiService.getAuditorByOpenid(auditLogQO.getOpenid(), oldAuditLog.getEnterpriseId());
            if (!auditorServiceResponse.isSuccess()) {
                return ResultControllerUtils.commonResult(auditorServiceResponse);
            }
            auditorDTO = auditorServiceResponse.getResult();
        } else if (userDetail != null) {
            if (userDetail.getUserInfo().getSuperAdmin() == 0) {
                //后端审核的情况，一定是关联管理员的审核员
                ServiceResponse<AuditorDTO> auditorServiceResponse = auditorApiService.getAuditorByUserId(userDetail.getEnterpriseId(), userDetail.getUserId());
                if (!auditorServiceResponse.isSuccess()) {
                    return ResultControllerUtils.commonResult(auditorServiceResponse);
                }
                auditorDTO = auditorServiceResponse.getResult();
            } else {
                auditorDTO = new AuditorDTO();
                //超管没有审核员ID
                auditorDTO.setAuditorId(null);
                auditorDTO.setAuditorName(userDetail.getUserInfo().getUserName());
                auditorDTO.setPhone(userDetail.getUserInfo().getPhoneNumber());
            }
        } else {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数错误");
        }

        auditLogDTO.setAuditorName(auditorDTO.getAuditorName());
        auditLogDTO.setAuditorPhone(auditorDTO.getPhone());
        auditLogDTO.setAuditorId(auditorDTO.getAuditorId());

        ServiceResponse<AuditLogDTO> serviceResponse = auditLogApiService.audit(auditLogDTO);
        if (serviceResponse.isSuccess()) {
            createLog(auditLogDTO);
            return RestResponse.success();
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage(), serviceResponse.getResult());
        }
    }

    private void createLog(AuditLogDTO auditLogDTO) {
        ServiceResponse<AuditLogDTO> auditLog = this.auditLogApiService.getAuditLog(auditLogDTO.getAuditLogId());
        LogUtils.createLog("审核审批项", auditLog.getResult().getProjectItemName(), auditLogDTO.getAuditorName(),
                auditLog.getResult().getEnterpriseId(), auditLog.getResult().getEnterpriseName(), "+86", auditLogDTO.getAuditorPhone());
    }

    @RequestMapping("/get-audit-log")
    @IgnoreLogin
    public RestResponse getAuditLog(Integer auditLogId) {
        ServiceResponse<AuditLogDTO> serviceResponse = this.auditLogApiService.getAuditLog(auditLogId);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/list-audit-log")
    public RestResponse listAuditLog(AuditLogQO auditLogQO) {
        auditLogQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        auditLogQO.setUserId(UserDetailUtils.getUserDetail().getUserId());
        if (UserDetailUtils.getUserDetail().getUserInfo().getSuperAdmin() == 1) {
            auditLogQO.setAdminUser(true);
        }
        ServiceResponse<Page<AuditLogDTO>> serviceResponse = this.auditLogApiService.listAuditLog(auditLogQO);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/close-audit-log")
    public RestResponse close(Integer auditLogId) {
        return ResultControllerUtils.commonResult(auditLogApiService.closeAuditLog(auditLogId));
    }

    @RequestMapping("/project-item-info")
    public RestResponse projectItemInfo(String projectItemCode) {
        ServiceResponse<ProjectItemDTO> serviceResponse = this.projectItemApiService.getByProjectItemCode(projectItemCode);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/ensure-audit")
    public RestResponse ensureAudit(String projectItemCode) {
        // 是否需要审批  需要审批true   不需要审批false
        if (UserDetailUtils.getUserDetail().getUserInfo().getSuperAdmin() == 1) {
            return RestResponse.success(false);
        }
        ServiceResponse<ProjectItemDTO> serviceResponse = this.projectItemApiService.getByProjectItemCode(projectItemCode);
        if (!serviceResponse.isSuccess()) {
            return ResultControllerUtils.commonResult(serviceResponse);
        }
        return RestResponse.success(serviceResponse.getResult().getUseStatus() == 1);
    }

}
