package com.gic.enterprise.web.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.annotation.HeaderSignIgnore;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.ano.IgnoreLogin;
import com.gic.enterprise.dto.CustomGuideDTO;
import com.gic.enterprise.dto.GuideTypeDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.CustomGuideApiService;
import com.gic.enterprise.utils.UserDetailUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;
/**
 *
 * @ClassName: CustomGuideController
 * @Description:
 * @author taogs
 * @date 2020/9/11 15:28
 */
@RestController
public class CustomGuideController {
    private static final Logger log = LogManager.getLogger(CustomGuideController.class);
    @Autowired
    private CustomGuideApiService customGuideApiService;

    @RequestMapping("save-custom-guide")
    public RestResponse saveCustomGuide(String param, boolean isNeedNewVersion, String appId){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        JSONArray array = JSON.parseArray(param);
        List<CustomGuideDTO> list = new ArrayList<>();
        Map<Integer, Object> exist = new HashMap<>();
        Map<String, Object> pagePathExist = new HashMap<>();
        ServiceResponse<List<GuideTypeDTO>> response = this.customGuideApiService.listGuideType();
        for(int i=0; i< array.size(); i++){
            JSONObject json = array.getJSONObject(i);
            int customLinkType = json.getIntValue("customLinkType");
            if(customLinkType != 0 && customLinkType != 7){
                if(exist.get(customLinkType) != null){
                    return RestResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "链接类型不能重复选择");
                }
                exist.put(customLinkType, customLinkType);
            }
            CustomGuideDTO guideDTO = new CustomGuideDTO();
            guideDTO.setGuideId(json.getInteger("guideId"));
            guideDTO.setIcon(json.getString("icon"));
            guideDTO.setTitle(json.getString("title"));
            guideDTO.setEntryCondition(json.getInteger("entryCondition"));
            guideDTO.setEnterpriseId(enterpriseId);
            guideDTO.setLink(json.getString("link"));
            guideDTO.setCustomLinkType(customLinkType);
            guideDTO.setSort(i);
            guideDTO.setStatus(json.getInteger("status"));
            guideDTO.setAppid(appId);
            guideDTO.setIconPath(json.getString("iconPath"));
            guideDTO.setSelectedIconPath(json.getString("selectedIconPath"));
            this.getPath(guideDTO, list, response.getResult());
            if(pagePathExist.get(guideDTO.getCustomPage()) != null){
                return RestResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "链接路径不能重复选择");
            }
            pagePathExist.put(guideDTO.getCustomPage(), guideDTO.getCustomPage());
            list.add(guideDTO);
        }
        if(!isNeedNewVersion && this.isNeedNewVersion(list, enterpriseId, appId)){
            return RestResponse.failure(ErrorCode.ISNEEDNEWVERSION.getCode(), ErrorCode.ISNEEDNEWVERSION.getErrorMsg());
        }
        log.info("保存的导航内容:{}", JSON.toJSONString(list));
        ServiceResponse<Void> serviceResponse = this.customGuideApiService.saveGuide(list, isNeedNewVersion, enterpriseId, appId);
        if(serviceResponse.isSuccess()){
            return RestResponse.success();
        }else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("list-custom-guide")
    public RestResponse listCustomGuide(String appId){
        ServiceResponse<List<CustomGuideDTO>> response = this.customGuideApiService.listGuide(UserDetailUtils.getUserDetail().getEnterpriseId(), appId);
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("list-guide-type")
    public RestResponse listGuideType(){
        ServiceResponse<List<GuideTypeDTO>> response = this.customGuideApiService.listGuideType();
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("get-ext-json")
    @IgnoreLogin
    @HeaderSignIgnore
    public RestResponse getExtJson(Integer enterpriseId, String appid){
        ServiceResponse<Map<String, Object>> extJson = this.customGuideApiService.getExtJson(enterpriseId, appid);
        return RestResponse.success(extJson.getResult());
    }

    private boolean isNeedNewVersion(List<CustomGuideDTO> list, Integer enterpriseId, String appid){
        ServiceResponse<List<CustomGuideDTO>> response = this.customGuideApiService.listGuide(enterpriseId, appid);
        List<CustomGuideDTO> oldList = response.getResult();
        if(CollectionUtils.isEmpty(oldList)){
            return true;
        }
        if(list.size() != oldList.size()){
            return true;
        }
        String newUrl = "";
        for(CustomGuideDTO dto : list){
            newUrl += dto.getCustomPage();
        }
        String oldUrl = "";
        for(CustomGuideDTO dto : oldList){
            oldUrl += dto.getCustomPage();
        }
        if(newUrl.equals(oldUrl)){
            return false;
        }
        return true;
    }

    private void getPath(CustomGuideDTO guideDTO, List<CustomGuideDTO> list, List<GuideTypeDTO> guideTypeDTOList){
        Integer linkType = 7;
        if(guideDTO.getCustomLinkType() == 0){ //没有选择类型
            if(StringUtils.isNotBlank(guideDTO.getLink())){
                JSONObject jsonObject = JSON.parseObject(guideDTO.getLink());
                guideDTO.setCustomPage(jsonObject.getString("url"));
            }
        }else if(guideDTO.getCustomLinkType().equals(linkType)){ //类型为其他
            List<String> listCustomPage = Arrays.asList("pages/customView/customView1","pages/customView/customView2",
                    "pages/customView/customView3","pages/customView/customView4","pages/customView/customView5");
            guideDTO.setCustomPage("pages/customView/customView"+ (guideDTO.getSort().intValue()+1));
            /**for(String path : listCustomPage){
                boolean exist = false;
                for(CustomGuideDTO dto : list){
                    if(path.equals(dto.getCustomPage())){
                        exist = true;
                    }
                }
                if(!exist){
                }
            }*/
        }else { //类型选择为1-6
            for(GuideTypeDTO guideTypeDTO : guideTypeDTOList){
                if(guideDTO.getCustomLinkType().equals(guideTypeDTO.getGuideType())){
                    guideDTO.setCustomPage(guideTypeDTO.getPath());
                }
            }
        }
    }
}
